#!/usr/bin/env python3

# create a new public_html/index.html file

import re, os
import html

public_html = os.path.join( os.environ['HOME'], 'public_html/' )
data_folder = os.path.join( public_html, 'data/' )

videos = []

file_re = re.compile('^(.*)\.([^\.]*)$')
# read the files in public_html/data
for filename in os.listdir( data_folder ):
  print("Checking " + filename)
  tmp = file_re.match( filename )
  base = tmp.group(1)
  ext = tmp.group(2)
  if ext == "mp4" or ext == "webm":
    data = {}
    data["filename"] = filename
    data["base"] = base
    data["extension"] = ext
    sub_file = os.path.join( data_folder, base + ".vtt" )
    print( " Checking for subs in " + sub_file )
    if os.path.exists( sub_file ):
      data["subtitles"] = base + ".vtt"
    #link_file = os.path.join( data_folder, "linkto_" + base )
    #print( " Checking for links in " + link_file )
    #if os.path.exists( link_file ):
    #  data["link"] = "linkto_" + base
    
    videos.append(data)
    
# order videos
videos = sorted( videos, key = lambda el : el["filename"] )

# write the new index.html file
index_html = os.path.join( public_html, "index.html" )
f = open( index_html, "w" )
head = '''<!DOCTYPE html>

<html>

<head>
<title>My shared page</title>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
<link rel="icon" type="image/png" href="share/favicon.png">
<style>
  body{
    background-image: url('share/bg/background_main.png'), url('share/bg/background_base.png');
    background-repeat: no-repeat, repeat;
    background-position: right bottom, left top;
    background-attachment: fixed, scroll;
  }
  .title_div{
    display: block;
  }
  .title{
    display: block;
    font-weight:bold;
    font-size: 16pt;
    text-align: center;
  }
  .notice_div{
    display: block;
    padding: 10px 10px;
    color: #666666;
  }
  .menu_div{
    display: block;
    padding: 10px 10px;
  }
  .menu_head{
    font-weight:bold;
    font-size: 12pt;
  }
  .videos_div{
    display: block;
  }
  video {
    display: block;
    width: 600px;
    height: 300px;
    /*border: 1px solid black;*/
  }
  .video_div{
    max-height: 600px;
    padding: 10px 10px;
  }
  .filename{
    font-weight:bold;
  }
</style>
</head>

<body>
<div class="title_div"><p class="title">My Streaming Page</p></div>
<div class="notice_div">Notice: View this site using Chromium version >= 38 or Firefox version >= 26</div>
'''

f.write(head)

f.write('<div class="menu_div"><p class="menu_head">MENU</p>')
f.write('<p class="menu_items">')
# menu
for video in videos:
  data = '<a href="#id_' + html.escape(video["filename"]) + '">' + html.escape(video["filename"]) + '</a><br>'
  f.write(data)

f.write('</p></div>')
  
# videos
f.write('<div id="videos" class="videos_div">')
for video in videos:
  if video["extension"] == "mp4":
    video_codec = 'video/mp4; codecs="avc1.42E01E, mp4a.40.2"'
  else:
    video_codec = 'video/webm; codecs="vp8, vorbis"'
  
  video_data = '<div id="id_' + html.escape(video["filename"]) + '" class="video_div">'
  video_data = video_data + '<p class="filename">' + html.escape(video["base"])
  #if "link" in video:
  #  video_data = video_data + ' <a href="' + html.escape( video["link"] ) + '">Download</a>'
  video_data += '</p>'
  video_data += '<video poster="share/poster.png" controls>'
  video_data = video_data + '<source src="data/' + html.escape(video["filename"]) + '" type=\'' + video_codec + '\'>'
  if "subtitles" in video:
    video_data = video_data + '<track src="data/' + html.escape(video["subtitles"]) +'" kind="subtitles" srclang="en" label="English" default></track>'
  
  video_data = video_data + 'Video tag not supported. <a href="data/' + html.escape(video["filename"]) + '">Download the video</a>.</video></div>'

  f.write(video_data)

f.write('</div>')

# close
tail = '''
<!--<div class="download_items">
<p>All files <a href="data/">HERE<a></p>
</div>-->
</body>

</html>
'''

f.write(tail)

f.close()
