#!/usr/bin/env python

from SimpleHTTPServer import SimpleHTTPRequestHandler
import os
import mimetypes
import urllib
import time

class SimpleRequestHandler( SimpleHTTPRequestHandler ):

  def send_basic_file_headers( self ):
    # sends the following headers:
    # Last-Modified
    # Date
    
    stats = os.stat( self.path )
    last_modified = time.strftime("%a, %d %b %Y %H:%M:%S %Z", time.localtime(stats.st_mtime) )
    # GMT format : time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime(stats.st_mtime) )
    now_date = time.strftime("%a, %d %b %Y %H:%M:%S %Z", time.localtime() )
    
    # send the last-modified header ( useful so the browser does not need to download the same data twice )
    self.send_header( 'Last-Modified', last_modified )
    self.send_header( 'Date', now_date )
    
  def do_GET(self):
    
    root = "/home/francesco/public_html"
    self.path = os.path.join( root, self.path[1:] )
    self.path = urllib.unquote( self.path )
    
    #print( "Serving: " + self.path )
    
    try:
      
      # folder
      if os.path.isdir( self.path ):
        index = os.path.join( self.path, "index.html" )
        if os.path.exists( index ):
          self.path = index
        else:
          # list folder
          # TODO
          self.send_error(404,'File Not Found: %s' % self.path)
        
          return
      
      # generic file
      if os.path.exists( self.path ):
        
        # html
        if self.path.endswith(".html"):
          f = open( self.path, "r" )
          
          # send headers
          self.send_response(200)
          file_size = os.path.getsize(self.path)
          self.send_header( 'Content-Type',	'text/html')
          self.send_header( 'Content-Length', file_size )
          self.send_basic_file_headers( )
          self.end_headers()
          
          # send file
          self.wfile.write( f.read() )
          f.close()
          
          return
        
        else:
          # other file
          mime_data = mimetypes.guess_type( self.path )
          mime = mime_data[0]
          if mime == None:
            # guess mime TODO
            mime = "text/plain"
          
          encoding = mime_data[1]
          file_size = os.path.getsize(self.path)
          
          # read range
          if "Range" in self.headers:
            tmp = self.headers["Range"].split("=")
            #range_tmp = self.headers["Range"][6:]
            # I assume tmp[0] == "bytes"
            range_tmp = tmp[1]
            tmp = range_tmp.split("-")
            range_start = int( tmp[0] )
            range_end = int( tmp[1] ) if tmp[1] != "" else file_size - 1
            partial = True
          else:
            range_start = 0
            range_end = file_size - 1
            partial = False
          
          MAX_SIZE = 1048576 # 1 MB each transfer (the size is in bytes)
          content_length = range_end - range_start + 1
          
          if content_length > MAX_SIZE:
            # split forcefully the data
            range_end = range_start + MAX_SIZE - 1
            content_length = range_end - range_start + 1
            partial = True
          
          '''
          print( "Mime: " + mime )
          print( "Size: " + str( file_size ) )
          print( "Range: " + str(range_start) + " - " + str( range_end ) )
          print( "Content-Length: " + str( content_length ) )
          '''
          
          if partial:
            self.send_response( 206 )
          else:
            self.send_response( 200 )
          
          self.send_header( 'Content-Type', mime )
          self.send_header( 'Content-Length', str( content_length ) )
          self.send_basic_file_headers( )
          
          if partial:
            range_res = "bytes " + str(range_start) + "-" + str(range_end) + "/" + str(file_size)
            #print( 'Content-Range: ' + range_res )
            self.send_header( 'Content-Range', range_res )
          
          self.end_headers()
          
          # open file and send
          f = open( self.path, "r" )
          f.read( range_start )
          self.wfile.write( f.read( content_length ) )
          f.close()
          
          return
          
      else:
        # file not found
        self.send_error(404,'File Not Found: %s' % self.path)
        
        return
    
    except IOError:
      self.send_error(404,'File Not Found: %s' % self.path)    

