#!/bin/bash

# Stream the given file
# NOTE: I assume simple_server.py is already started

## functions
function findFormatMKV {

  # NOTE: Should work for other containers too, but needs work
  
  local mkv_data=`mkvmerge -i "$file"`
  local mkv_audio=""
  local mkv_video=""
  local mkv_tmp=""
  
  mkv_tmp=`echo "$mkv_data" | grep video -m 1 | cut -c 10-`
  mkv_video=`echo "$mkv_tmp" | grep -e '(.*)' -o`
  
  # video: recognise mpeg4 and theora
  if [ "$mkv_video" == "(V_MPEG4/ISO/AVC)" ] ||  [ "$mkv_video" == "(avc1)" ]; then
    video_format="mp4"
  elif [ "$mkv_video" == "(V_THEORA)" ] || [ "$mkv_video" == "(Theora)" ]; then
    video_format="theora"
  else
    echo "WARNING: Video format $mkv_video unknown."
    video_format=$mkv_video
  fi
  
  mkv_tmp=`echo "$mkv_data" | grep audio -m 1 | cut -c 10-`
  mkv_audio=`echo "$mkv_tmp" | grep -e '(.*)' -o`
  
  # audio: recognise aac and theora
  if [ "$mkv_audio" == "(A_AAC)" ] || [ "$mkv_audio" == "(mp4a)" ]; then
    audio_format="aac"
  elif [ "$mkv_audio" == "(A_VORBIS)" ] || [ "$mkv_audio" == "(Vorbis)" ]; then
    audio_format="vorbis"
  else
    echo "WARNING: Audio format $mkv_audio unknown."
    audio_format=$mkv_audio
  fi
  
}
main_folder=$( dirname "${BASH_SOURCE[0]}" )

file=$1
format=$2
force=$3

# input check
if [ ! -e "$file" ] ; then
  echo "File $file does not exists. Aborting."
  exit 1
fi

if [ "$format" == "" ]; then
  format="ogv"
fi

if [ $format != "mp4" ] && [ $format != "ogv" ]; then
  echo "Format $format unknown. Aborting."
  exit 1
fi

# setup file variables
filename=$(basename "$file")
extension="${filename##*.}"
filename="${filename%.*}"

# FORMAT SETUP

# original format
if [ $extension == "mkv" ]; then
  findFormatMKV
else
  audio_format="???"
  video_format="???"
fi

# destination format
if [ $format == "ogv"  ]; then
  dest_audio="vorbis"
  dest_video="theora"
  video_encoder="libtheora"
  audio_encoder="libvorbis"
else
  dest_audio="aac"
  dest_video="mp4"
  video_encoder="libx264"
  audio_encoder="aac"
fi

# encoder options
# video
if [ "$video_format" == "$dest_video" ]; then
  video_encoder="copy"
fi

# audio
if [ "$audio_format" == "$dest_audio" ]; then
  audio_encoder="copy"
fi

# strict?
if [ "$audio_encoder" == "aac" ]; then
  avconv_option="-strict experimental"
else
  avconv_option=""
fi

# size warning
filesize=`du "$file" | grep -e "[0-9]*" -o | head -n 1`
if [ "$force" != "force" ] && [ $video_encoder != "copy" ] && [ $filesize -gt 10240 ]; then
  # Files greater than 10 MB will not be converted
  echo "File is too large, PC may overheat. Aborting."
  exit 2
fi

# setup dest variables
dest_folder="/home/francesco/public_html/data"
dest_file="$dest_folder/$filename.$format"
dest_subs="$dest_folder/$filename.vtt"
#dest_link="$dest_folder/linkto_$filename"

# convert file
echo "Converting $filename to $format"
echo "Video: $video_encoder"
echo "Audio: $audio_encoder"
echo "Options: $avconv_option"
echo "avconv -i "$file" "$avconv_option" -c:v "$video_encoder" -c:a "$audio_encoder" "$dest_file 
avconv -i "$file" $avconv_option -c:v "$video_encoder" -c:a "$audio_encoder" "$dest_file"

# try to extract subs
if [ $extension == "mkv" ]; then
  "$main_folder/bin/extract_subs_vtt.sh" "$file" "$dest_subs"
fi

# create a link to the original file in public_html/data
#ln -s -T "$file" "$dest_link"

# add an entry to index.html
"$main_folder/bin/generate_index.py"
